/*
 * Copyright 2017 Marc Liberatore.
 */

package sequencer;

import java.util.List;

public class Assembler {

	/**
	 * Creates a new Assembler containing a list of fragments.
	 * 
	 * The list is copied into this assembler so that the original list will not
	 * be modified by the actions of this assembler.
	 * 
	 * @param fragments
	 */
	public Assembler(List<Fragment> fragments) {
	}

	/**
	 * Returns the current list of fragments this assembler contains.
	 * 
	 * @return the current list of fragments
	 */
	public List<Fragment> getFragments() {
		/*
		 */
	}

	/**
	 * Attempts to perform a single assembly, returning true iff an assembly was
	 * performed.
	 * 
	 * This method chooses the best assembly possible, that is, it merges the
	 * two fragments with the largest overlap, breaking ties between merged
	 * fragments by choosing the shorter merged fragment.
	 * 
	 * Merges must have an overlap of at least 1.
	 * 
	 * After merging two fragments into a new fragment, the new fragment is
	 * inserted into the list of fragments in this assembler, and the two
	 * original fragments are removed from the list.
	 * 
	 * @return true iff an assembly was performed
	 */
	// public boolean assembleOnce() {
	// return false;
	// }

	public boolean assembleOnce() {
		int largest = 0;
		int possible;
		int fragLength = 0;
		Fragment left = null;
		Fragment right = null;
		Fragment merger = null;
		for (int i = 0; i < fragments.size(); i++) {
			for (int j = 0; j < fragments.size(); j++) {
				if (j != i) {
					possible = fragments.get(i).calculateOverlap(fragments.get(j));
					if (possible > largest) {
						largest = possible;
						left = fragments.get(i);
						right = fragments.get(j);
						fragLength = left.mergedWith(right).length();
					}
					if (possible == largest) {
						if (fragLength > fragments.get(i).mergedWith(fragments.get(j)).length()) {
							left = fragments.get(i);
							right = fragments.get(j);
						}
					}
				}
			}
		}

		if (largest < 1) {
			return false;
		}

		if (largest >= 1) {
			merger = left.mergedWith(right);
		}

		fragments.add(merger);
		fragments.remove(left);
		fragments.remove(right);
		return true;
	}

	/**
	 * Repeatedly assembles fragments until no more assembly can occur.
	 */
	public void assembleAll() {
	}
}
